///////////////////////////////////////////////////////////////////////////////
// Vertex Shader
///////////////////////////////////////////////////////////////////////////////
#define _USE_WVP
#define _USE_DIFFUSEMAP

#include "StandardParameters.fxh"
#include "Effect_common.fxh"

// The output of the Vertex Shader
struct VS_OUTPUT
{
  // The transformed position
  float4 vPosition : POSITION;
 // The UV coordinates of the texture.
  float2 vTextureUV : TEXCOORD0;
};

///////////////////////////////////////////////////////////////////////////////
// simplest vertex shader just transforms and ignores normals
// @TODO, light support
VS_OUTPUT VS_TransformTex1(
  const float3 vPosition : POSITION,
  const float3 vNormal : NORMAL,
  const float2 vTextureUV : TEXCOORD0
  )
{
	// Transform the position from object space to homogeneous projection space
	VS_OUTPUT vsOut;
	vsOut.vPosition = ModelToScreenSpace( vPosition );
	vsOut.vTextureUV = vTextureUV;
	return vsOut;
}

///////////////////////////////////////////////////////////////////////////////
// Pixel Shader
///////////////////////////////////////////////////////////////////////////////

// Modulates the diffuse color of the vertex with the diffuse map
float4 PS_Diffuse( VS_OUTPUT psIn ): COLOR0
{
	// only the texture unlit right now
	return tex2D( g_sampDiffuse, psIn.vTextureUV );
}

///////////////////////////////////////////////////////////////////////////////
// default Effect
///////////////////////////////////////////////////////////////////////////////

// renders the object with just a diffuse texture
technique RenderDiffuse
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";

// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_DEFAULT;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// render the diffuse texture
	pass DiffusePass
	{
		AlphaBlendEnable = true;
		ZFunc = LESSEQUAL;
		ZEnable = true;
		ZWriteEnable = false;
		SrcBlend = ONE;
		DestBlend = INVSRCALPHA;
		VertexShader = compile vs_3_0 VS_TransformTex1( );
		PixelShader = compile ps_3_0 PS_Diffuse();
	}
}

// Default lighting.
#include "FixedPipelineDiffuse.fxh"

///////////////////////////////////////////////////////////////////////////////
