///////////////////////////////////////////////////////////////////////////////
// Vertex Shader
///////////////////////////////////////////////////////////////////////////////

#define _USE_WVP
#define _USE_DIFFUSEMAP
#define _USE_MATERIAL
#define _USE_CLOAKING


#include "StandardParameters.fxh"
#include "Effect_common.fxh"

///////////////////////////////////////////////////////////////////////////////
// this value must exactly match the enum value of

// The output of the Vertex Shader
struct VS_OUTPUT
{
  // The transformed position
  float4 vPosition : POSITION;
  // The UV coordinates of the texture.
  float2 vDiffuseUV : TEXCOORD0;
};

///////////////////////////////////////////////////////////////////////////////

// simplest vertex shader just transforms and ignores normals
VS_OUTPUT VS_TransformedTex1(
  const float3 vPosition : POSITION,
  const float3 vNormal : NORMAL,
  const float2 vTextureUV : TEXCOORD0)
{
	VS_OUTPUT vsOut;
	vsOut.vPosition = ModelToScreenSpace( vPosition );
	vsOut.vDiffuseUV = vTextureUV;
	
	// if cloaking is enabled, this object needs to be distorted so that its depth value is 
    // written correctly into the depth buffer; the cloaking effect expects the depth buffer 
    // to be established before it begins its processing - jmg
    if (g_bEnableCloakDistortion)
    {
	    // distort the model in screen space - jmg
		vsOut.vPosition.xy += ComputeCloakingDistortion(vsOut.vPosition.xy, g_fDistortionWaveAmplitude, g_CloakTime);
    }
	return vsOut;
}

///////////////////////////////////////////////////////////////////////////////
// Pixel Shader
///////////////////////////////////////////////////////////////////////////////


// Modulates the diffuse color of the vertex with the diffuse map
float4 PS_SelfIllumination( VS_OUTPUT psIn ): COLOR0
{
	float4 cDiffTex = tex2D( g_sampDiffuse, psIn.vDiffuseUV );
	
	// we have to ignore the texture alpha because of data set irregularities
	// we just premultiply the colors
	cDiffTex.rgb *= g_cMaterialAmbient;
	return float4(cDiffTex.rgb,1); //*g_cMaterialDiffuse.a; - removed to fix cloaking; is this necessary? - jmg
}

///////////////////////////////////////////////////////////////////////////////
// default Effect
///////////////////////////////////////////////////////////////////////////////

// renders the object with just a diffuse texture
technique RenderGlow
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = 1;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// render the diffuse texture
	pass DiffusePass
	{
		VertexShader = compile vs_3_0 VS_TransformedTex1();
		PixelShader = compile ps_3_0 PS_SelfIllumination();
	}
}

///////////////////////////////////////////////////////////////////////////////
technique RenderFallback
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = 0;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
}
