#define _USE_WVP
#define _USE_MATERIAL
#define _USE_DIFFUSEMAP

#include "StandardParameters.fxh"
#include "Effect_common.fxh"

/// sampler declaration for nebula colume texture
#ifdef _WIN32
texture g_texNebulaCloud : Mad3D_VolumeTexture;
//------------------------------------
sampler3D g_sampCloud = sampler_state 
{
    texture = <g_texNebulaCloud>;
    AddressU = WRAP;
    AddressV = WRAP;
    AddressW = WRAP;
    MIPFILTER = POINT;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};
#else
sampler3D g_sampCloud  : Mad3D_VolumeTexture 
= sampler_state 
{
    AddressU = WRAP;
    AddressV = WRAP;
    AddressW = WRAP;
    MIPFILTER = POINT;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
    };
#endif


struct VS_OUTPUT 
{
    float4 pPosition	: POSITION;
    float4 cColor		: COLOR0;
    float2 tcDiffuse	: TEXCOORD0;
    float3	tcNoise		: TEXCOORD1;
};

VS_OUTPUT VS_Nebula( float3 pPosition : POSITION,
											float4 cDiffuse : COLOR0,
											float2 tcDiffuse : TEXCOORD0 )
{
    VS_OUTPUT OUT;

    OUT.pPosition = ModelToScreenSpace( pPosition );
    OUT.tcDiffuse = tcDiffuse;
    OUT.cColor = cDiffuse;
    // map into 0-1 range
    OUT.tcNoise = (pPosition/g_tcVolumeTextureScale)/2+0.5;
 	return OUT;
}


///////////////////////////////////////////////////////////////////////////////
// Diffuse: DiffTexColor * (DiffLightColor * (N . L) + AmbLightColor)
// The final color is diffuse.
float4 PS_Nebula( VS_OUTPUT IN ): COLOR
{
	float4 c1 = tex2D( g_sampDiffuse,IN.tcDiffuse );
	// its really grey scale ...
	if( c1.r <= (float)( 1.0f/255.0f )
	/*&& c1.g <= (float)( 1.0f/255.0f )
	&& c1.b <= (float)( 1.0f/255.0f )*/ )
	{
		return float4( 0, 0, 0, 0) ;
	}
	else
	{
		float3 c0 = tex3D( g_sampCloud, IN.tcNoise ).rgb;
		float4 cColor = c0.r*g_cMaterialDiffuse+(1-c0.g)*g_cMaterialAmbient;
		// The effective diffuse texture color 
		return float4( IN.cColor.rgb * c1.rgb * cColor, IN.cColor.a*c1.a*c0.g );
	}
}


struct VS_OUTPUT_LOW
{
    float4 pPosition	: POSITION;
    float4 cColor		: COLOR0;
    float2 tcDiffuse	: TEXCOORD0;
};

VS_OUTPUT_LOW VS_Nebula_Low( float3 pPosition : POSITION,
											float4 cDiffuse : COLOR0,
											float2 tcDiffuse : TEXCOORD0 )
{
    VS_OUTPUT_LOW OUT;

    OUT.pPosition = ModelToScreenSpace( pPosition );
    OUT.tcDiffuse = tcDiffuse;
    OUT.cColor = cDiffuse;
 	return OUT;
}
float4 PS_Nebula_Low( VS_OUTPUT_LOW IN ): COLOR
{
	float4 c1 = tex2D( g_sampDiffuse,IN.tcDiffuse );
	float4 cColor = (g_cMaterialDiffuse+g_cMaterialAmbient)/2;
	// The effective diffuse texture color 
	return float4( IN.cColor.rgb * c1.rgb * cColor, IN.cColor.a*c1.a );
}

// renders the object with just a diffuse texture and a volume texture
technique Render
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosColorTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_LOW;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	pass BasePass
	{
		AlphaBlendEnable = true;
		AlphaTestEnable = true;
		AlphaFunc = greater;
		AlphaRef = 0;
		ZFunc = LESSEQUAL;
		ZEnable = true;
		ZWriteEnable = false;
		SrcBlend = ONE;
		DestBlend = INVSRCALPHA;
		CullMode = CW;

		VertexShader = compile vs_3_0 VS_Nebula( );
		PixelShader = compile ps_3_0 PS_Nebula();
	}
}

// renders the object with just a diffuse texture
technique RenderLow
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosColorTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_FALLBACK;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	pass BasePass
	{
		AlphaBlendEnable = true;
		AlphaTestEnable = true;
		AlphaFunc = greater;
		AlphaRef = 0;
		ZFunc = LESSEQUAL;
		ZEnable = true;
		ZWriteEnable = false;
		SrcBlend = ONE;
		DestBlend = INVSRCALPHA;
		CullMode = CW;
		VertexShader = compile vs_3_0 VS_Nebula_Low( );
		PixelShader = compile ps_3_0 PS_Nebula_Low();
	}
}


// renders the object with just a diffuse texture
technique Rendernothing
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosColorTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_LASTRESORT;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
}