///////////////////////////////////////////////////////////////////////////////
// Vertex Shader
///////////////////////////////////////////////////////////////////////////////
#define _USE_WVP
#define _USE_MATERIAL
#define _USE_DIFFUSEMAP

#include "StandardParameters.fxh"
#include "effect_common.fxh"

/// sampler declaration for nebula colume texture
#ifdef _WIN32
texture g_texNebulaCloud : Mad3D_VolumeTexture;
//------------------------------------
sampler3D g_sampCloud = sampler_state 
{
    texture = <g_texNebulaCloud>;
    AddressU = WRAP;
    AddressV = WRAP;
    AddressW = WRAP;
    MIPFILTER = POINT;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};
#else
sampler3D g_sampCloud  : Mad3D_VolumeTexture 
= sampler_state 
{
    AddressU = WRAP;
    AddressV = WRAP;
    AddressW = WRAP;
    MIPFILTER = POINT;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
    };
#endif


struct VS_OUTPUT
{
  // The transformed position
  float4 vPosition : POSITION;
  // The diffuse color 
  float4 cDiffuse : COLOR0;
 // The UV coordinates of the texture.
  float2 tcDiffuse : TEXCOORD0;
  float3	tcNoise		: TEXCOORD1;
};

///////////////////////////////////////////////////////////////////////////////
VS_OUTPUT VS_TransformedTex(
  const float3 vPosition : POSITION,
  const float3 vNormal : NORMAL,
  const float3 tcDiffuse : TEXCOORD0,
  uniform float4 cColorDiffuse )
{
	VS_OUTPUT vsOut;
	vsOut.vPosition = ModelToScreenSpace( vPosition );

	// output constant diffuse color
	vsOut.cDiffuse = cColorDiffuse;

	// output texture coordinates
	vsOut.tcDiffuse = tcDiffuse;
    // map into 0-1 range
    vsOut.tcNoise = (vPosition/g_tcVolumeTextureScale)/2+0.5;
	return vsOut;
}

float4 PS_Diffuse( VS_OUTPUT psIn ): COLOR0
{
	float3 c0 = tex3D( g_sampCloud, psIn.tcNoise ).rgb;
	// light the texture 
	return tex2D( g_sampDiffuse, psIn.tcDiffuse )*psIn.cDiffuse*c0.g;
}

// renders the object with just a diffuse texture
technique Render
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_DEFAULT;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// render the diffuse texture
	pass DiffusePass1
	{
		AlphaTestEnable = true;
		AlphaFunc = always;
		AlphaRef = 0;
		AlphaBlendEnable = true;
		ZFunc = ALWAYS;
		ZEnable = false;
		ZWriteEnable = false;
		SrcBlend = ONE;
		DestBlend = INVSRCALPHA;
		CullMode = CW;
		VertexShader = compile vs_3_0 VS_TransformedTex( g_cMaterialDiffuse );
		PixelShader = compile ps_3_0 PS_Diffuse();
	}
}


struct VS_OUTPUT_LOW
{
    float4 pPosition	: POSITION;
    float4 cColor		: COLOR0;
    float2 tcDiffuse	: TEXCOORD0;
};

VS_OUTPUT_LOW VS_LOW(
  const float3 vPosition : POSITION,
  const float3 vNormal : NORMAL,
  const float3 tcDiffuse : TEXCOORD0,
  uniform float4 cColorDiffuse )
{
	VS_OUTPUT_LOW vsOut;
	vsOut.pPosition = ModelToScreenSpace( vPosition );

	// output constant diffuse color
	vsOut.cColor = cColorDiffuse;

	// output texture coordinates
	vsOut.tcDiffuse = tcDiffuse;
	return vsOut;
}

float4 PS_LOW( VS_OUTPUT_LOW psIn ): COLOR0
{
	// light the texture 
	return tex2D( g_sampDiffuse, psIn.tcDiffuse )*psIn.cColor;
}

// renders the object with just a diffuse texture
technique Render
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_LOW;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// render the diffuse texture
	pass DiffusePass1
	{
		AlphaTestEnable = true;
		AlphaFunc = always;
		AlphaRef = 0;
		AlphaBlendEnable = true;
		ZFunc = ALWAYS;
		ZEnable = false;
		ZWriteEnable = false;
		SrcBlend = ONE;
		DestBlend = INVSRCALPHA;
		CullMode = CW;
		VertexShader = compile vs_3_0 VS_LOW( g_cMaterialDiffuse );
		PixelShader = compile ps_3_0 PS_LOW();
	}
}
// no prelight in worse case where card does no 3d texturing.
technique Render
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_FALLBACK;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
}
