

// bright pass just extracts luminance from a scene past a threshold, black below, color over
// use to adjust a scene before we do a bloom pass

#include "postprocess.fxh"




// thresholf fed by the engine
float Luminance : Mad3D_BloomThreshold = 0.08f;
static const float fMiddleGray = 0.18f;
static const float fWhiteCutoff = 0.8f;

///////////////////////////////////////////////////////////////////////////////
// this value must exactly match the enum value of

// The output of the Vertex Shader
struct VS_OUTPUT
{
  // The transformed position
  float4 vPosition : POSITION;
  float4 cDiffuse : COLOR0;
  // The UV coordinates of the texture.
  float2 vDiffuseUV : TEXCOORD0;
};

///////////////////////////////////////////////////////////////////////////////

// simplest vertex shader just transforms and ignores normals
VS_OUTPUT VS_PassThrough(
  const float4 vPosition : POSITION,
  const float4 cColor : COLOR0,
  const float2 vTextureUV : TEXCOORD0
  )
{
	VS_OUTPUT vsOut;
	vsOut.vPosition = vPosition;
	vsOut.cDiffuse = cColor;
	vsOut.vDiffuseUV = vTextureUV;
	return vsOut;
}

// Perform a high-pass filter on the source texture
float4 BrightPassFilter( in float2 Tex : TEXCOORD0 ) : COLOR0
{
    float3 ColorOut = tex2D( g_samSrcColor, Tex );

    ColorOut *= fMiddleGray / ( Luminance + 0.001f );
    ColorOut *= ( 1.0f + ( ColorOut / ( fWhiteCutoff * fWhiteCutoff ) ) );
    ColorOut -= 5.0f;

    ColorOut = max( ColorOut, 0.0f );

    ColorOut /= ( 10.0f + ColorOut );

    return float4( ColorOut, 1.0f );
}

technique PostProcess
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosRHWColorTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = 1;
// the number of lights the effect can do, we just ignore them
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
    pass p0
    {
 		CullMode = none;
	   	SrcBlend = one;
    	DestBlend = zero;
        VertexShader = compile vs_3_0 VS_PassThrough();
        PixelShader = compile ps_3_0 BrightPassFilter();
        ZEnable = false;
    }
}


technique PostProcess
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosRHWColorTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = 0;
// the number of lights the effect can do, we just ignore them
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
}
