
// color glow, samples the src on an horizontal direction using a sampling kernel
// outputs an horizontally blured value
#include "postprocess.fxh"



static const int g_cKernelSize = 9;
float2 PixelKernel[g_cKernelSize] =
{
//    { -6, 0 },
//    { -5, 0 },
    { -4, 0 },
    { -3, 0 },
    { -2, 0 },
    { -1, 0 },
    {  0, 0 },
    {  1, 0 },
    {  2, 0 },
    {  3, 0 },
    {  4, 0 },
//    {  5, 0 },
//    {  6, 0 },
};

float2 TexelKernel[g_cKernelSize]
<
    string ConvertPixelsToTexels = "PixelKernel";
>;

/*
static const float BlurWeights[g_cKernelSize] = 
{
    0.002216,
    0.008764,
    0.026995,
    0.064759,
    0.120985,
    0.176033,
    0.199471,
    0.176033,
    0.120985,
    0.064759,
    0.026995,
    0.008764,
    0.002216,
};
*/


static const float BlurWeights[g_cKernelSize] = 
{
//    0.001,
//    0.005,
    0.01,
    0.03,
    0.07,
    0.15,
    0.3,
    0.15,
    0.07,
    0.03,
    0.01,
//    0.005,
//    0.001,
};

///////////////////////////////////////////////////////////////////////////////
// this value must exactly match the enum value of

// The output of the Vertex Shader
struct VS_OUTPUT
{
  // The transformed position
  float4 vPosition : POSITION;
  float4 cDiffuse : COLOR0;
  // The UV coordinates of the texture.
  float2 vDiffuseUV : TEXCOORD0;
};

///////////////////////////////////////////////////////////////////////////////

// simplest vertex shader just transforms and ignores normals
VS_OUTPUT VS_PassThrough(
  const float4 vPosition : POSITION,
  const float4 cColor : COLOR0,
  const float2 vTextureUV : TEXCOORD0
  )
{
	VS_OUTPUT vsOut;
	vsOut.vPosition = vPosition;
	vsOut.cDiffuse = cColor;
	vsOut.vDiffuseUV = vTextureUV;
	return vsOut;
}

float fBloomScale : Mad3D_GlowFactor = 1.9f;

//-----------------------------------------------------------------------------
// Pixel Shader: HorizontalBlur
// Desc: Blurs the image horizontally
//-----------------------------------------------------------------------------
float4 PostProcessPS( float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 Color = 0;

    for (int i = 0; i < g_cKernelSize; i++)
    {    
        Color += tex2D( g_samSrcColor, Tex + TexelKernel[i].xy ) * BlurWeights[i];
    }

    return Color * fBloomScale;
}



technique PostProcess
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosRHWColorTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = 1;
// the number of lights the effect can do, we just ignore them
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
    pass p0
    {
		CullMode = none;
		SrcBlend = one;
    	DestBlend = zero;
        VertexShader = compile vs_3_0 VS_PassThrough();
        PixelShader = compile ps_3_0 PostProcessPS();
        ZEnable = false;
    }
}

technique PostProcess
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosRHWColorTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = 0;
// the number of lights the effect can do, we just ignore them
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
}