
#define _USE_P
#define _USE_BACKBUFFER
#define _USE_ELAPSEDTIME

// the implementation is in this file
#include "StandardParameters.fxh"
#include "Effect_common.fxh"

static const float blurSamples : samples
<
    string UIWidget = "slider";
    float UIMin = 1.0;
    float UIMax = 32.0;
    float UIStep = 1.0;
    string UIName = "blur samples";
> = 8.0f;

float blurStrength : blur
<
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 0.1;
    float UIStep = 0.001;
    string UIName = "blur strength";
> = 0.025f;

float aspectRatio
<
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 2.0;
    float UIStep = 0.01;
    string UIName = "aspect ratio";
> = 1.333f;

//------------------------------------
struct vertexInput2 {
    float3 position				: POSITION;
    float3 normal				: NORMAL;
    float4 texCoordDiffuse		: TEXCOORD0;
};

struct vertexOutput2 {
    float4 hPosition		: POSITION;
    float4 texCoordDiffuse	: TEXCOORD0;
    float2 texVelocity		: TEXCOORD1;
};


//------------------------------------
vertexOutput2 VS_TransformAndTexture(vertexInput2 IN, 
									 uniform float inElapsedTime) 
{
    vertexOutput2 OUT;
    
    // scale the input time to provide a faster transformation
    float localTime = min(inElapsedTime * 2, 1.0f);
    
    // c
    OUT.hPosition = float4(IN.position.xy, 5.0f, 1.0f);    
    
    // move the vertex toward the viewer to create a tunnel effect
    OUT.hPosition.z -= (max(abs(IN.position.x), abs(IN.position.y * aspectRatio)) * pow(localTime, 1)) * 5;
    
    // project the point in screen-space and provide it with a depth
    // value that will not clip against the view frustum
    OUT.hPosition = mul(OUT.hPosition, g_mProjection);
    OUT.hPosition.z = 0.5f;
    
    // compute the velocity of the vertex
    OUT.texVelocity = normalize(OUT.hPosition.xy) * blurStrength * localTime;
    OUT.texVelocity.x = -OUT.texVelocity.x;
    
    // pass the texture coordinates through
    OUT.texCoordDiffuse = IN.texCoordDiffuse;    

    return OUT;
}


//-----------------------------------
float4 PS_Textured(vertexOutput2 IN): COLOR
{
    float2 pixelVelocity;
    float3 Blurred = 0;
    for(int i = 0; i < blurSamples; i++)
    {   
    	// sample the texture in the spot scaled by the current sample
    	// and the velocity vector and average it with the other samples
        float2 texelDisplacement = IN.texVelocity * i / blurSamples + IN.texCoordDiffuse.xy;
        
        // look-up the texel located at the computed position in the back buffer
        float4 currentTexel = tex2D(g_sampBackBuffer, texelDisplacement);
        
        // add the current texel value to the running sum
        Blurred += currentTexel.rgb;
    }
    
    // average the combined samples and return the result
    return float4(Blurred / blurSamples, 1.0f);
}


//-----------------------------------
technique screenMotionBlur
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosRHWColorTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_DEFAULT;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
    pass p0 
    {
		VertexShader = compile vs_3_0 VS_TransformAndTexture(g_fElapsedTime);
		PixelShader  = compile ps_3_0 PS_Textured();
    }
}

technique screenMotionBlur_not_supported
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosRHWColorTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_LASTRESORT;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
}