



// depth bias

float4 shadowConst = float4( 0.00000002f, 1.0f, 0.0000002f, 4 );
#define SHADOW_EPSILON shadowConst.x 
#define SHADOW_SCALEBIAS shadowConst.y
#define SHADOW_MAXGRADIENT shadowConst.z
#define SHADOW_16SAMPLE_KERNEL_WIDTH shadowConst.w

// parameters for filtering
float g_TexelSize : Mad3D_ShadowMapTexelSize = 1/1024;
float g_TextureSize : Mad3D_ShadowMapSize = 1024;


#ifdef _WIN32
texture g_texGaussianMap : Mad3D_GaussianFilter;
sampler g_sampGaussianFilter = sampler_state 
{
    texture = <g_texGaussianMap>;
    AddressU  = CLAMP;
    AddressV  = CLAMP;
    MIPFILTER = LINEAR;
    MINFILTER = LINEAR;
    MAGFILTER = POINT;
};
#else
sampler g_sampGaussianFilter : Mad3D_GaussianFilter
= sampler_state 
{
};
#endif






// shadow projected transform
float4 WorldSpaceToShadowSpace( float4 WorldPos )
{
	// hack, shadow matrix in view inverse param
	// scale to improve z precision
	return  mul( WorldPos, g_mViewInverse );
}

// take projected pos and returns UV for shadow map sample
float2 ClipSpaceToUVSpace( float4 clipSpace )
{
        float2 uv = 0.5 * clipSpace.xy /* clipSpace.w*/ + float2( 0.5, 0.5 );
        uv.y = 1.0f - uv.y;
        return uv;
}


// returns a shadowmap depth value
float ShadowDepthSample( float2 tc, float2 Offset )
{
	float tex = tex2D( g_sampShadow1, tc+Offset*g_TexelSize ).r;
	// not rendered there so not in shadow
	if( tex == 0 )
	{
		return 1;
	}
	else
	{
		return tex;
	}
}


// samples shadow once and return 0 OR 1 depending on depth compare
// returns 0 or 1
float OneShadowSample( float2 tc, float pixeldepth )
{
	float tex = tex2D( g_sampShadow1, tc ).r;
	if( tex == 0 )
	{
		return 1;
	}
	else
	{
		return (tex + SHADOW_EPSILON < pixeldepth)? 0.0f: 1.0f;
	}
}

// the offset is in PIXEL space
// same as above but takes an offset
// returns 0 or 1
float ShadowSample( float2 tc, float2 Offset, float pixeldepth )
{
	float tex = ShadowDepthSample( tc, Offset );
	return ( tex + SHADOW_EPSILON < pixeldepth)? 0.0f: 1.0f;
}

/*
// sample 4 times and lerp the result
float FourShadowSamplePCF( float2 tc, float pixeldepth )
{
    // transform to texel space
    float2 texelpos = g_TextureSize * tc;
       
    // Determine the lerp amounts           
    float2 lerps = frac( texelpos );

	float D = pixeldepth;
	// shadow
	float4 depths = float4( ShadowDepthSample( tc, float2(-1, 0) ),
							ShadowDepthSample( tc, float2(1, 0) ),
							ShadowDepthSample( tc, float2(0, -1) ),
							ShadowDepthSample( tc, float2(0, 1) ) );
	
	float2 diff = abs( depths.yw - depths.xy );
	float gradient  = SHADOW_EPSILON + SHADOW_SCALEBIAS* max( diff.x, diff.y );
	
	float4 shadows = float4( depths.x+gradient < D ? 0.0f: 1.0f,
							depths.y+gradient < D ? 0.0f: 1.0f,
							depths.z+gradient < D ? 0.0f: 1.0f,
							depths.w+gradient < D ? 0.0f: 1.0f );
		
		
	
	float shadow = lerp( lerp( shadows.x, shadows.y, lerps.x ),
                                  lerp( shadows.z, shadows.w, lerps.x ), lerps.y );
    return shadow;



//	return ShadowDepthSample( tc, float2( 0,0) ) + gradient > D;
}
*/
// sample 4 times and lerp the result
float FourShadowSamplePCF( float2 tc, float D )
{
	// shadow
	float4 sourcevals = float4( ShadowSample( tc, float2(0,0), D ),
								ShadowSample( tc, float2(1, 0), D ),
								ShadowSample( tc, float2(0, 1), D ),
								ShadowSample( tc, float2(1, 1), D ) );
								
    // transform to texel space
    float2 texelpos = g_TextureSize * tc;
       
    // Determine the lerp amounts           
    float2 lerps = frac( texelpos );
    	
	float shadow = lerp( lerp( sourcevals.x, sourcevals.y, lerps.x ),
                                  lerp( sourcevals.z, sourcevals.w, lerps.x ),
                                  lerps.y );
    return shadow;
}
// takes a projected UV and computes shadows
// soft edges, but bad shadow acne at sharp angles
float ComputeShadow4PCF( float4 tc )
{
    float2 ShadowTexC = ClipSpaceToUVSpace( tc );
		
	// sample 4 times and lerp
	return FourShadowSamplePCF( ShadowTexC, tc.z /*tc.w*/ );
}

// this is a hard edged version that behaves better in regards to shadow acne
float Compute4SampleInterpolatedDepthShadow( float4 tc )
{
    float2 ShadowTexC = ClipSpaceToUVSpace( tc );
    
	// depth of current pixel
	float D = tc.z;
	
	// shadow
	float shadowDepth = 0;
	shadowDepth += ShadowDepthSample( ShadowTexC, float2(0,0) );
	shadowDepth += ShadowDepthSample( ShadowTexC, float2(1, 0) );
	shadowDepth += ShadowDepthSample( ShadowTexC, float2(0, 1) );
	shadowDepth += ShadowDepthSample( ShadowTexC, float2(1, 1) );
	shadowDepth/= 4;
    return (shadowDepth +SHADOW_EPSILON < D)? 0.0f: 1.0f;
}

float ShadowSample16( float4 tc )
{
 	float4 constVals  = float4( -1.5f, g_TextureSize/SHADOW_16SAMPLE_KERNEL_WIDTH, 0.125f, 1.5f );
    float2 ShadowTexC = ClipSpaceToUVSpace( tc );
    
    float2 base = floor( g_TextureSize * ShadowTexC );
	float2 pos;
	float2 curCoord;
    float D = tc.z;

	// loop sampling 16 times
	float u = constVals.x;
	float v = constVals.x;
	float2 shadow = 0;
	
	for( pos.x = constVals.x; pos.x<=constVals.w; pos.x++ )
	{
		for( pos.y = constVals.x; pos.y<=constVals.w; pos.y++ )
		{
			float2 currCoord = (base + pos + 0.5) / g_TextureSize;
			shadow  += float2( OneShadowSample( currCoord, D ).x , 1 ) *  tex2D( g_sampGaussianFilter, (ShadowTexC-currCoord )*constVals.y + 0.5 ).r;
		}
	}
	
/*	// we unroll to be able to do slope scale bias
	// sample corners to get gradient
	float4 depths = float4( ShadowDepthSample( tc, float2(-1.5, 0) ),
							ShadowDepthSample( tc, float2(1.5, 0) ),
							ShadowDepthSample( tc, float2(0, -1.5) ),
							ShadowDepthSample( tc, float2(0, 1.5) ) );
	
	float2 diff = abs( depths.yw - depths.xy );
	float gradient  = SHADOW_EPSILON + SHADOW_SCALEBIAS*min( SHADOW_MAXGRADIENT, max( diff.x, diff.y ) );*/
	
	return shadow.x/shadow.y;
}