#define _USE_WVP
#define _USE_W
#define _USE_SHIELD_FX
#define _USE_ELAPSEDTIME

// the implementation is in this file
#include "StandardParameters.fxh"
#include "Effect_common.fxh"

struct diffuseSpecularVertexOutput {
    float4 pScreenPosition	: POSITION;
    float3 pWorldPosition	: TEXCOORD0;
    float3 pModelPosition	: TEXCOORD1;
};

diffuseSpecularVertexOutput VS_BeamShieldImpact(vertexInput IN)
{
	diffuseSpecularVertexOutput OUT;

	// the pixel shader needs the vertex position in model, screen, and world space
    OUT.pScreenPosition = ModelToScreenSpace(IN.pPosition); 
	OUT.pWorldPosition 	= ModelToWorldSpace(IN.pPosition);
	OUT.pModelPosition 	= IN.pPosition;
	
	return OUT;
}


// this annotation is a standard annotation used by mad3d to generate
// a noise texture as requested
// on PC its on the texture on xbox its the sampler
#ifdef _WIN32
texture g_texShieldBeam
<
 string TextureName = "ShieldBeam";
 int PerlinScale = 2;
 int TextureSize = 256;
 int TextureDepth = 1;
 int TextureFormat = TEXTUREFORMAT_CUBE;
 int NoiseType = TEXTURE_FIRE;
 int Octaves = 2;
>;
sampler g_sampShieldBeam = sampler_state 
{
    texture = <g_texShieldBeam>;
    AddressU  = <g_DiffuseU>;
    AddressV  = <g_DiffuseV>;
    MIPFILTER = <g_MipFilter>;
    MINFILTER = <g_MinFilter>;
    MAGFILTER = <g_MagFilter>;
};
#else // XBOX
sampler g_sampShieldBeam 
<
 string TextureName = "ShieldBeam";
 int PerlinScale = 2;
 int TextureSize = 128;
 int TextureDepth = 1;
 int TextureFormat = TEXTUREFORMAT_CUBE;
 int NoiseType = TEXTURE_FIRE;
 int Octaves = 2;
>
= sampler_state 
{
    AddressU  = WRAP;
    AddressV  = WRAP;
    MIPFILTER = LINEAR;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};
#endif
///////////////////////////////////////////////////////////////////////////////

float4 PS_BeamShieldImpact(diffuseSpecularVertexOutput IN): COLOR
{	
	float3 	finalColor;
	
	// compute the distance from the point of impact
	float3 vDistanceFromImpact = (IN.pWorldPosition - g_fShieldImpactPoint);
	
	// scale the distance by the shield coverage
	vDistanceFromImpact /= g_fShieldCoverage * 2;
	
	// compute the intensity of the pixel based on the distance from impact
	float fIntensity = length(vDistanceFromImpact);	
	fIntensity = 1.0f - fIntensity;
	
	// sample the volumetric perlin noise texture at the specified location
	float4 noiseAmount = texCUBE(g_sampShieldBeam, IN.pModelPosition);
	
	// sharpen the edges of the perlin noise texture
	noiseAmount = pow(noiseAmount, 2);
	
	// increase the intensity of the perlin noise; this creates a sharper look
	noiseAmount *= 4;
	
	// clamp the perlin noise to a valid range
	noiseAmount.r = min(noiseAmount.r, 1.0f);
	
	// linearly ping-pong the intensity of the perlin noise to produce
	// an electric-like effect
	noiseAmount.r = ((noiseAmount.r + g_fElapsedTime) % 1.4f);	
	noiseAmount.r = max(noiseAmount.r, 0.6f);	
	
	if (noiseAmount.r > 1.0f)
	{
		noiseAmount.r = 2.0f - noiseAmount.r;
	}
	
	// blend the perlin noise texture with the intensity of the shield
	finalColor = noiseAmount.r * (g_fShieldColor * fIntensity);
	//finalColor = noiseAmount.r * (float3(0.0f, 0.4f, 0.7f) * fIntensity);
	
	// highlight the area around the point of impact
	finalColor.xyz += max(pow(fIntensity, 3), 0.0f);
	
	// return the final color
	return float4(finalColor.rgb, fIntensity * noiseAmount.r * g_fShieldColor.a);
}

///////////////////////////////////////////////////////////////////////////////

technique shieldBeam
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_LOW;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
    pass BasePass
    {
    	AlphaBlendEnable 	= true;
		ZFunc 				= LESSEQUAL;
		ZEnable 			= true;
		ZWriteEnable 		= false;
		SrcBlend 			= srcalpha;
		DestBlend 			= invsrcalpha;
    	CullMode 			= cw;
		VertexShader 		= compile vs_3_0 VS_BeamShieldImpact();
		PixelShader  		= compile ps_3_0 PS_BeamShieldImpact();
    }
}

///////////////////////////////////////////////////////////////////////////////

// Default lighting.
#include "FixedPipelineDiffuse.fxh"

///////////////////////////////////////////////////////////////////////////////
