#define _USE_WVP
#define _USE_W
#define _USE_SHIELD_FX
#define _USE_ELAPSEDTIME

// the implementation is in this file
#include "StandardParameters.fxh"
#include "Effect_common.fxh"

struct diffuseSpecularVertexOutput {
    float4 pScreenPosition	: POSITION;
    float3 pWorldPosition	: TEXCOORD0;
    float3 pModelPosition	: TEXCOORD1;
};

diffuseSpecularVertexOutput VS_BulletShieldImpact(vertexInput_ST IN)
{
	diffuseSpecularVertexOutput OUT;

	// the pixel shader needs the vertex position in model, screen, and world space
    OUT.pScreenPosition = ModelToScreenSpace(IN.pPosition); 
	OUT.pWorldPosition 	= ModelToWorldSpace(IN.pPosition);
	OUT.pModelPosition 	= IN.pPosition;
	
	return OUT;
}


// this annotation is a standard annotation used by mad3d to generate
// a noise texture as requested
#ifdef _WIN32
texture g_texShieldBullet
<
 string TextureName = "ShieldBullet";
 int PerlinScale = 3;
 int TextureSize = 256;
 int TextureDepth = 1;
 int TextureFormat = TEXTUREFORMAT_CUBE;
 int NoiseType = TEXTURE_FIRE;
 int Octaves = 5;
>;
sampler g_sampShieldBullet = sampler_state 
{
    texture = <g_texShieldBullet>;
    AddressU  = <g_DiffuseU>;
    AddressV  = <g_DiffuseV>;
    MIPFILTER = <g_MipFilter>;
    MINFILTER = <g_MinFilter>;
    MAGFILTER = <g_MagFilter>;
};
#else // XBOX
sampler g_sampShieldBullet 
<
 string TextureName = "ShieldBullet";
 int PerlinScale = 3;
 int TextureSize = 128;
 int TextureDepth = 1;
 int TextureFormat = TEXTUREFORMAT_CUBE;
 int NoiseType = TEXTURE_FIRE;
 int Octaves = 5;
>
= sampler_state 
{
    AddressU  = WRAP;
    AddressV  = WRAP;
    MIPFILTER = LINEAR;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};
#endif
///////////////////////////////////////////////////////////////////////////////

float4 PS_BulletShieldImpact(diffuseSpecularVertexOutput IN): COLOR
{
	// compute the distance from the point of impact
	float3 vDistanceFromImpact = (IN.pWorldPosition - g_fShieldImpactPoint);
	
	// scale the distance by the shield coverage
	vDistanceFromImpact /= g_fShieldCoverage;
	
	// the intensity of the shield for this fragment is tied 
	// to its distance from the point of impact	
	float fIntensity = length(vDistanceFromImpact);
	
	// grow the shield from its impact point
	fIntensity += 1.0f - g_fShieldElapsedTime;
	
	// deteriorate the shield from its impact point
	fIntensity = abs(fIntensity - (g_fShieldElapsedTime * 2));
	
	// invert the intensity
	fIntensity = 1.0f - fIntensity;
	
	// use the interpolated model position of the vertex 
	// to acquire the amount of noise at this fragment location
	float4 noiseAmount = texCUBE(g_sampShieldBullet, IN.pModelPosition);

	// blend the shield gradient with the shield noise texture
	float3 finalColor = g_fShieldColor * fIntensity;
	finalColor += noiseAmount.r * fIntensity * 0.3;
	
	// highlight the ring
	finalColor.xyz += max(pow(fIntensity, 3), 0.0f) / 2;
	
	// modulate the shield color by the amount of time elapsed to 
	// make the shield fade away over time
	fIntensity *= 1.0f - g_fShieldElapsedTime;

	// return the final color
	return float4(finalColor.rgb, fIntensity * g_fShieldColor.a);
}

///////////////////////////////////////////////////////////////////////////////

technique shieldBullet
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_LOW;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
    pass BasePass
    {
    	AlphaBlendEnable 	= true;
		ZFunc 				= LESSEQUAL;
		ZEnable 			= true;
		ZWriteEnable 		= false;
		SrcBlend 			= srcalpha;
		DestBlend 			= invsrcalpha;
    	CullMode 			= cw;
		VertexShader 		= compile vs_3_0 VS_BulletShieldImpact();
		PixelShader  		= compile ps_3_0 PS_BulletShieldImpact();
    }
}

///////////////////////////////////////////////////////////////////////////////

// Default lighting.
#include "FixedPipelineDiffuse.fxh"

///////////////////////////////////////////////////////////////////////////////
