///////////////////////////////////////////////////////////////////////////////
// INPUT
///////////////////////////////////////////////////////////////////////////////
#define _USE_WVP
#define _USE_DIFFUSEMAP
#include "Standardparameters.fxh"


///////////////////////////////////////////////////////////////////////////////
// Vertex Shader
///////////////////////////////////////////////////////////////////////////////


// The output of the Vertex Shader
struct VS_OUTPUT
{
  // The position of the star in projection space.
  float4 vPosition : POSITION;
  // The diffuse color of the star.
  float4 vDiffuse : COLOR0;
  // The UV coordinates of the first texture.
  float2 vTextureUV : TEXCOORD0;
};

VS_OUTPUT VS_TransformNoLightTex1(
  const float4 vPosition : POSITION,
  const float4 vDiffuse : COLOR0,
  const float2 vTextureUV : TEXCOORD0)
{
  VS_OUTPUT vsOut;

  vsOut.vPosition = mul( vPosition, g_mWorldViewProj );
  vsOut.vDiffuse = vDiffuse;
  vsOut.vTextureUV = vTextureUV;

  return vsOut;
}

///////////////////////////////////////////////////////////////////////////////
// Pixel Shader
///////////////////////////////////////////////////////////////////////////////


float4 PS_Modulate( VS_OUTPUT psIn ) :COLOR0
{
	// modulate 2X is used by the stars
	return tex2D( g_sampDiffuse, psIn.vTextureUV ) * psIn.vDiffuse * 2.0f;
}

///////////////////////////////////////////////////////////////////////////////
// Starfield Effect
///////////////////////////////////////////////////////////////////////////////


technique RenderStarfield
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosColorTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = 1;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
  // We only require one pass.
  pass p0
  {
    ZEnable = false;
	ZWriteEnable = false;
	CullMode = NONE;
	AlphaBlendEnable = true;
	DestBlend = ONE;
	SrcBlend = ONE;
    VertexShader = compile vs_3_0 VS_TransformNoLightTex1();
    PixelShader = compile ps_3_0 PS_Modulate();
  }
}

