///////////////////////////////////////////////////////////////////////////////
// Simple screen geometry techniques for vertex having been 
// processed with software rendering.
///////////////////////////////////////////////////////////////////////////////
#define _USE_DIFFUSEMAP
#include "StandardParameters.fxh"

///////////////////////////////////////////////////////////////////////////////
// this value must exactly match the enum value of

// The output of the Vertex Shader
struct VS_OUTPUT
{
  // The transformed position
  float4 vPosition : POSITION;
  float4 cDiffuse : COLOR0;
  // The UV coordinates of the texture.
  float2 vDiffuseUV : TEXCOORD0;
};
	
///////////////////////////////////////////////////////////////////////////////

// simplest vertex shader just transforms and ignores normals
VS_OUTPUT VS_PassThrough(
  const float4 vPosition : POSITION,
  const float4 cColor : COLOR0,
  const float2 vTextureUV : TEXCOORD0
  )
{
	VS_OUTPUT vsOut;
	vsOut.vPosition = vPosition;
	vsOut.cDiffuse = cColor;
	vsOut.vDiffuseUV = vTextureUV;
	return vsOut;
}
VS_OUTPUT VS_PassThrough2(
  const float4 vPosition : POSITION,
  const float4 cColor : COLOR0,
  const float4 cColor1 : COLOR1,
  const float2 vTextureUV : TEXCOORD0
  )
{
	VS_OUTPUT vsOut;
	vsOut.vPosition = vPosition;
	vsOut.cDiffuse = cColor*cColor1;
	vsOut.vDiffuseUV = vTextureUV;
	return vsOut;
}

///////////////////////////////////////////////////////////////////////////////
// Pixel Shader
///////////////////////////////////////////////////////////////////////////////

// Modulates the diffuse color of the vertex with the diffuse map
float4 PS_Diffuse( VS_OUTPUT psIn ): COLOR0
{
	float4 cDiffTex = tex2D( g_sampDiffuse, psIn.vDiffuseUV );
	//return float4(1.0f, 1.0f, 1.0f, 1.0f);
	return cDiffTex * psIn.cDiffuse;
}
// renders the object with just a diffuse texture
technique RenderDiffuseTex
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosRHWColorTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = 0;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// render the diffuse texture
	pass DiffuseTex
	{
		StencilEnable = TRUE;
		StencilFunc = EQUAL;
		StencilRef = 1;
		StencilMask = 1;
		StencilPass = KEEP;
		VertexShader = compile vs_3_0 VS_PassThrough();
		PixelShader = compile ps_3_0 PS_Diffuse();
	}
}

// renders the object with just a diffuse texture
technique RenderScreenGeometry2
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosRHWColor2Tex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = 0;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// render the diffuse texture
	pass DiffusePass
	{
		StencilEnable = TRUE;
		StencilFunc = LESSEQUAL;
		StencilRef = 1;
		StencilMask = 1;
		StencilPass = KEEP;
		VertexShader = compile vs_3_0 VS_PassThrough();
		PixelShader = compile ps_3_0 PS_Diffuse();
	}
}
