
#define _USE_W
#define _USE_WTrans
#define _USE_WVP
#define _USE_NORMALMAP
#define _USE_BACKBUFFER
#define _USE_LIGHTS
#define _USE_EYEPOS
#define _USE_CLOAKING
#define _USE_ELAPSEDTIME

// the implementation is in this file
#include "StandardParameters.fxh"
#include "Effect_common.fxh"


// constants that define the look of the portal
float g_fTwistAmount 			= 0.01f;
float g_fDiffuseMapScrollSpeed 	= 0.2f;
float g_fNormalMapScrollSpeed 	= 0.08f;
float g_fModelSize 				= 160.0f;
float g_fDistortionIntensity 	= 0.5f;

///////////////////////////////////////////////////////////////////////////////

struct screenDistortionVertexOutput {
    float4 pPosition		: POSITION;
    float3 tcDiffuse		: TEXCOORD0;
    float2 tcNormal			: TEXCOORD1;
    float2 tcSpecular		: TEXCOORD2;
    float3 tcIncident		: TEXCOORD3;
};

///////////////////////////////////////////////////////////////////////////////

screenDistortionVertexOutput VS_DistortBackground(vertexInput_ST IN,
												  uniform float3 vCameraPos,
												  uniform float inAmplitude,
												  uniform float inTime)
{
	screenDistortionVertexOutput OUT;
	float4 vPositionModel;
	float3 vCameraModel;
	float3 vIncidentModel;

	// transform the vertex and camera position from world space to model space
	vPositionModel 	 = float4(IN.pPosition.xyz, 1.0f);
    vCameraModel.xyz = vCameraPos.xyz - g_mWorld[3];
    
    // compute the camera-to-vertex vector in normal space and pack it
    vIncidentModel = vPositionModel - vCameraModel;
    vIncidentModel = WorldToModelSpaceRotate(vIncidentModel);
    vIncidentModel = normalize(vIncidentModel);
    OUT.tcIncident = float4(ToNormalSpace(vIncidentModel, IN.vNormal, IN.vTangent, 
    									  IN.vBinormal), 0.0f);
   
	// compute the alpha value to soften the edge of the portal - jmg
    OUT.tcDiffuse.z = g_fModelSize - IN.pPosition.y;
    OUT.tcDiffuse.z = clamp(OUT.tcDiffuse.z, 0.0f, 1.0f);
    
    // rotate the texture coordinates based on the vertex's position from the model's
    // origin; this creates a twist in the texture - jmg 
    OUT.tcDiffuse.xy = IN.tcDiffuse;
    OUT.tcDiffuse.x += IN.pPosition.y * g_fTwistAmount; 
    
    // use the diffuse texture coordinates for the normal and specular maps - jmg
	OUT.tcNormal 	= IN.tcDiffuse;
	OUT.tcSpecular 	= IN.tcDiffuse;
	
    // scroll the diffuse and normal textures outward - jmg
    OUT.tcDiffuse.y -= inTime * g_fDiffuseMapScrollSpeed;
    OUT.tcNormal.y  -= inTime * g_fNormalMapScrollSpeed;   
		
	// transform the vertex position from model to screen space
    OUT.pPosition = ModelToScreenSpace(IN.pPosition);

	return OUT;
}

///////////////////////////////////////////////////////////////////////////////
float4 PS_DistortBackground(screenDistortionVertexOutput IN,
							uniform float inTime): COLOR
{
	float4 vNormal;
	float4 vBackBufferColor;
	float3 vIncident;
	float  fresnelFactor;	
	
	// fetch the packed normal from the normal map and unpack it accordingly
	vNormal = tex2D(g_sampNormal, IN.tcNormal);
	vNormal = Unpack(vNormal);
		
	// normalize the unpacked normal and incident vectors
	vNormal   = normalize(vNormal);
	vIncident = normalize(IN.tcIncident.xyz);
	
	// compute the fresnel coefficient
    fresnelFactor = pow(1.0f + dot(vIncident.xyz, vNormal.xyz), 1.0f);
    
   	// sample the back buffer
   	//fresnelFactor *= g_cCloakColor.a;
   	float2 texcoord = IN.tcDiffuse.xy;
	vBackBufferColor = tex2D(g_sampBackBuffer, texcoord + (fresnelFactor * g_fDistortionIntensity));
    
	// return the final color
	return float4(vBackBufferColor.rgb, IN.tcDiffuse.z);
}

///////////////////////////////////////////////////////////////////////////////

technique transwarpPortal
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1ST";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_DEFAULT;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{    
    // this pass uses the frame buffer as a texture and distorts the model in screen
    // space to achieve a transition between being cloaked and uncloaked
    pass DistortPass
    {
    	AlphaBlendEnable 	= true;
		ZFunc 				= LESSEQUAL;
		ZEnable 			= true;
		ZWriteEnable 		= false;
		SrcBlend 			= SRCALPHA;
		DestBlend 			= INVSRCALPHA;
	
		VertexShader 		= compile vs_3_0 VS_DistortBackground(g_pEye, g_fDistortionWaveAmplitude, g_fElapsedTime);
		PixelShader  		= compile ps_3_0 PS_DistortBackground(g_fElapsedTime);
    }
}

///////////////////////////////////////////////////////////////////////////////

// Default lighting.
#include "FixedPipelineDiffuse.fxh"

///////////////////////////////////////////////////////////////////////////////
