///////////////////////////////////////////////////////////////////////////////

#define _USE_WVP
#include "StandardParameters.fxh"
#include "Effect_common.fxh"


// The output of the Vertex Shader
struct VS_OUTPUT
{
  // The transformed position
  float4 vPosition : POSITION;
  float3 fDepth : TEXCOORD0;
};

///////////////////////////////////////////////////////////////////////////////
// output z and w for depth
VS_OUTPUT VS_Z
(
  const float3 vPosition : POSITION,
  const float3 vNormal : NORMAL,
  const float2 vTextureUV : TEXCOORD0
  )
{
	VS_OUTPUT vsOut;
	vsOut.vPosition = ModelToScreenSpace( vPosition );
	// make the z value linear, is our far plane
	vsOut.fDepth = float3( vsOut.vPosition.z/DEPTH_SCALE, vsOut.vPosition.w/DEPTH_SCALE, 0 );
	return vsOut;
}

///////////////////////////////////////////////////////////////////////////////
// Pixel Shader
///////////////////////////////////////////////////////////////////////////////

// writes z in color buffer
float4 PS_Z( VS_OUTPUT psIn ): COLOR0
{
	return psIn.fDepth.x;// /psIn.fDepth.y;
}


technique RenderZFill
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = 1;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	pass ZFillPass
	{
		AlphaBlendEnable = false;
		ZEnable = true;
		ZWriteEnable = true;
		SrcBlend = ONE;
		DestBlend = ZERO;
		CullMode = ccw;
		VertexShader = compile vs_3_0 VS_Z( );
		PixelShader = compile ps_3_0 PS_Z();
	}
}

///////////////////////////////////////////////////////////////////////////////
technique RenderFallback
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = 0;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
}
