
#define _USE_WVP
#define _USE_CLOAKING

///////////////////////////////////////////////////////////////////////////////
// This shader just Z fills of a black color, partical to just to a first z fill pass, 
// the color write is because this is interleaved with other object that it must
// occlude correctly in the glow pass
///////////////////////////////////////////////////////////////////////////////

#include "StandardParameters.fxh"
#include "Effect_common.fxh"

///////////////////////////////////////////////////////////////////////////////
// this value must exactly match the enum value of

// The output of the Vertex Shader
struct VS_OUTPUT
{
  // The transformed position
  float4 vPosition : POSITION;
  float2 tcDiffuseTex : TEXCOORD0;
};

///////////////////////////////////////////////////////////////////////////////
// simplest vertex shader just transforms pos and ignore rest
VS_OUTPUT VS_Transform(
  const float3 vPosition : POSITION,
  const float3 vNormal : NORMAL,
  const float2 vTextureUV : TEXCOORD0
  )
{
	VS_OUTPUT vsOut;
	vsOut.vPosition = ModelToScreenSpace( vPosition );
	
	// if cloaking is enabled, this object needs to be distorted so that its depth value is 
    // written correctly into the depth buffer; the cloaking effect expects the depth buffer 
    // to be established before it begins its processing - jmg
    if (g_bEnableCloakDistortion)
    {
    	// distort the model in screen space - jmg
		vsOut.vPosition.xy += ComputeCloakingDistortion(vsOut.vPosition.xy, g_fDistortionWaveAmplitude, g_CloakTime);
	}
		
	vsOut.tcDiffuseTex = vTextureUV;
	return vsOut;
}

///////////////////////////////////////////////////////////////////////////////
// Pixel Shader
///////////////////////////////////////////////////////////////////////////////

// writes opaque black
float4 PS_Black( VS_OUTPUT psIn ): COLOR0
{
	return float4( 0,0,0,1);
}


technique RenderZFill
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = 1;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	pass ZFillPass
	{
		AlphaBlendEnable = false;
		VertexShader = compile vs_3_0 VS_Transform( );
		PixelShader = compile ps_3_0 PS_Black();
	}
}

///////////////////////////////////////////////////////////////////////////////
technique RenderFallback
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_LASTRESORT;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
}
