///////////////////////////////////////////////////////////////////////////////
// Vertex Shader
///////////////////////////////////////////////////////////////////////////////
#define _USE_WVP
#define _USE_DIFFUSEMAP
#define _USE_MATERIAL

#include "StandardParameters.fxh"
#include "effect_common.fxh"

///////////////////////////////////////////////////////////////////////////////
// this value must exactly match the enum value of
// The output of the Vertex Shader
struct VS_OUTPUT
{
  // The transformed position
  float4 vPosition : POSITION;
  // The diffuse color 
  float4 cDiffuse : COLOR0;
 // The UV coordinates of the texture.
  float2 tcDiffuse : TEXCOORD0;
};

///////////////////////////////////////////////////////////////////////////////

// simplest vertex shader just transforms and ignores normals
VS_OUTPUT VS_TransformedTex(
  const float3 vPosition : POSITION,
  const float3 vNormal : NORMAL,
  const float2 tcDiffuse : TEXCOORD0,
  uniform float4 cColorDiffuse,
  uniform float4 cColorAmbient
  )
{
	VS_OUTPUT vsOut;
	vsOut.vPosition = ModelToScreenSpace( vPosition );
	// output constant diffuse color
	vsOut.cDiffuse = cColorDiffuse + cColorAmbient;

	// output texture coordinates
	vsOut.tcDiffuse = tcDiffuse;
	return vsOut;
}


///////////////////////////////////////////////////////////////////////////////
// Pixel Shader
///////////////////////////////////////////////////////////////////////////////

// Modulates the diffuse color of the vertex with the diffuse map
float4 PS_Diffuse( VS_OUTPUT psIn ): COLOR0
{
	// light the texture
	return tex2D( g_sampDiffuse, psIn.tcDiffuse )*psIn.cDiffuse;
}

///////////////////////////////////////////////////////////////////////////////

// renders the object with just a diffuse texture
technique RenderFlatDiffuse
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_DEFAULT;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// render the diffuse texture
	pass DiffusePass1
	{
		VertexShader = compile vs_3_0 VS_TransformedTex( g_cMaterialDiffuse, g_cMaterialAmbient );
		PixelShader = compile ps_3_0 PS_Diffuse();
	}
}
///////////////////////////////////////////////////////////////////////////////

// Default lighting.
#include "FixedPipelineDiffuse.fxh"

///////////////////////////////////////////////////////////////////////////////
